@@ -94,41 +94,6 @@ class FlureeUnexpectedPredicateNumber(FlureeException):
         FlureeException.__init__(self, message)
 
 
-def _dryrun(method, url, headers, body):
-    """Helper function for debugging the library
-
-    Parameters
-    ----------
-    method :  str
-              HTTP method (POST or GET
-    url :     str
-              The url that would have been used
-    headers : dict
-              Dictionary with HTTP headers that would have been used
-    body :    str
-              The HTTP posting body
-    Returns
-    -------
-    dict
-        dummy dict, serves no purpose other than keeping to APIs
-    """
-    print("################################")
-    print("#            dryrun            #")
-    print("################################")
-    print("             ", method)
-    print("################################")
-    print("    ", url)
-    if headers is not None:
-        print("########### HEADERS ############")
-        for key in headers:
-            print(key, ":", headers[key])
-    if body is not None:
-        print("############ BODY ##############")
-        print(body)
-        print("################################")
-    return {"dryrun": True}
-
-
 _FLUREEQLQUERY_ENDPOINT_PERMISSIONS = {
     'query': {
         'permitted': {"select", "selectOne", "selectDistinct", "from", "where", "block", "prefixes", "vars", "opts"},
@@ -149,6 +114,54 @@ _FLUREEQLQUERY_ENDPOINT_PERMISSIONS = {
 }
 
 
+class _FlureeQlSubQuery:
+    """Helper class for FlureeQL multi-query syntactic sugar"""
+    def __init__(self, endpoint, method):
+        """Constructor
+
+        Parameters
+        ----------
+        endpoint : _FlureeQlEndpoint
+                   API endpoint for communicating FlureeQL queries with FlureeDB
+        method : str
+               Name for the sub-query
+        """
+        self.endpoint = endpoint
+        self.method = method
+        self.permittedkeys = _FLUREEQLQUERY_ENDPOINT_PERMISSIONS["query"]['permitted']
+        self.depricatedkeys = _FLUREEQLQUERY_ENDPOINT_PERMISSIONS["query"]['depricated']
+
+    def __call__(self, **kwargs):
+        """FlureeQl query construction through keyword arguments
+
+        Parameters
+        ----------
+        kwargs: dict
+                Keyword arguments for different parts of a FlureeQL query.
+
+        Raises
+        ------
+        TypeError
+            If an unknown kwarg value is used.
+
+        """
+        obj = {}
+        for key, value in kwargs.items():
+            if key == "ffrom":
+                key = "from"
+            if key == "ffilter":
+                key = "filter"
+            if key not in self.permittedkeys:
+                if key not in self.depricatedkeys:
+                    raise TypeError("FlureeQuery got unexpected keyword argument '" + key + "'")
+                print("WARNING: Use of depricated FlureeQL syntax,",
+                      key,
+                      "should not be used as top level key in queries",
+                      file=sys.stderr)
+            obj[key] = value
+        self.endpoint.multi_query[self.method] = obj
+
+
 class _FlureeQlQuery:
     """Helper class for FlureeQL query syntactic sugar"""
     def __init__(self, endpoint):
@@ -181,7 +194,7 @@ class _FlureeQlQuery:
         dict
             json decode result from the server.
         """
-        obj = dict()
+        obj = {}
         for key, value in kwargs.items():
             if key == "ffrom":
                 key = "from"
@@ -202,7 +215,8 @@ class _FlureeQlQuery:
 
         Parameters
         ----------
-        obj : Complete FlureeQl query object.
+        obj: dict
+            Complete FlureeQl query object.
 
         Returns
         -------
@@ -246,21 +260,18 @@ class _UnsignedGetter:
         FlureeHttpError
             If the server returns something different than a 200 OK status
         """
-        if self.session:
-            if self.ssl_verify_disabled:
-                async with self.session.get(self.url, ssl=False) as resp:
-                    if resp.status != 200:
-                        raise FlureeHttpError(await resp.text(), resp.status)
-                    response = await resp.text()
-                    return json.loads(response)
-            else:
-                async with self.session.get(self.url) as resp:
-                    if resp.status != 200:
-                        raise FlureeHttpError(await resp.text(), resp.status)
-                    response = await resp.text()
-                    return json.loads(response)
+        if self.ssl_verify_disabled:
+            async with self.session.get(self.url, ssl=False) as resp:
+                if resp.status != 200:
+                    raise FlureeHttpError(await resp.text(), resp.status)
+                response = await resp.text()
+                return json.loads(response)
         else:
-            return _dryrun("GET", self.url, None, None)
+            async with self.session.get(self.url) as resp:
+                if resp.status != 200:
+                    raise FlureeHttpError(await resp.text(), resp.status)
+                response = await resp.text()
+                return json.loads(response)
 
     async def ready(self):
         """Redo get untill ready condition gets met"""
@@ -272,11 +283,11 @@ class _UnsignedGetter:
                 obj = await self()
                 if obj[self.ready_field]:
                     return
-            except FlureeHttpError:
-                pass
+            except FlureeHttpError as ex:
+                print(ex)
             except aiohttp.client_exceptions.ClientConnectorError:
                 pass
-            await asyncio.sleep(0.5)
+            await asyncio.sleep(2)
 
 
 class _SignedPoster:
@@ -334,27 +345,24 @@ class _SignedPoster:
         FlureeHttpError
             When Fluree server returns a status code other than 200
         """
-        if self.session:
-            if self.ssl_verify_disabled:
-                async with self.session.post(self.url, data=body, headers=headers, ssl=False) as resp:
-                    if resp.status != 200:
-                        raise FlureeHttpError(await resp.text(), resp.status)
-                    data = await resp.text()
-                    try:
-                        return json.loads(data)
-                    except json.decoder.JSONDecodeError:
-                        return data
-            else:
-                async with self.session.post(self.url, data=body, headers=headers) as resp:
-                    if resp.status != 200:
-                        raise FlureeHttpError(await resp.text(), resp.status)
-                    data = await resp.text()
-                    try:
-                        return json.loads(data)
-                    except json.decoder.JSONDecodeError:
-                        return data
+        if self.ssl_verify_disabled:
+            async with self.session.post(self.url, data=body, headers=headers, ssl=False) as resp:
+                if resp.status != 200:
+                    raise FlureeHttpError(await resp.text(), resp.status)
+                data = await resp.text()
+                try:
+                    return json.loads(data)
+                except json.decoder.JSONDecodeError:
+                    return data
         else:
-            return json.loads(_dryrun("POST", self.url, headers, body))
+            async with self.session.post(self.url, data=body, headers=headers) as resp:
+                if resp.status != 200:
+                    raise FlureeHttpError(await resp.text(), resp.status)
+                data = await resp.text()
+                try:
+                    return json.loads(data)
+                except json.decoder.JSONDecodeError:
+                    return data
 
     async def __call__(self, **kwargs):
         """Invoke post API
@@ -376,7 +384,7 @@ class _SignedPoster:
         """
         # pylint: disable=too-many-locals
         kwset = set()
-        kwdict = dict()
+        kwdict = {}
         for key, value in kwargs.items():
             if not (key in self.required or key in self.optional):
                 raise TypeError("SignedPoster got unexpected keyword argument '" + key + "'")
@@ -497,15 +505,13 @@ class _DbFunctor:
         """
         return self.database
 
-    def __call__(self, privkey=None, auth_address=None, sig_validity=120, sig_fuel=1000):
+    def __call__(self, privkey=None, sig_validity=120, sig_fuel=1000):
         """Invoke functor
 
         Parameters
         ----------
         privkey : string
                   Private key for the specific DB.
-        auth_address : string
-                  Auth ID belonging with the privkey
         sig_validity : int
                        Validity in seconda of signatures.
         sig_fuel : int
@@ -516,37 +522,31 @@ class _DbFunctor:
             FlureeClient derived client for a specific DB
         """
         return _FlureeDbClient(privkey,
-                               auth_address,
                                self.database,
                                self.client.host,
                                self.client.port,
                                self.client.https,
                                self.client.ssl_verify,
                                sig_validity,
-                               sig_fuel,
-                               self.client.session is None)
+                               sig_fuel)
 
 
 class FlureeClient:
     """Basic asynchonous client for FlureeDB for non-database specific APIs"""
     def __init__(self,
                  masterkey=None,
-                 auth_address=None,
                  host="localhost",
                  port=8080,
                  https=False,
                  ssl_verify=True,
                  sig_validity=120,
-                 sig_fuel=1000,
-                 dryrun=False):
+                 sig_fuel=1000):
         """Constructor
 
         Parameters
         ----------
         masterkey : string
                   Hex or base58 encoded signing key
-        auth_address : string
-                       key-id of the signing key
         host : string
                    hostname of the FlureeDB server. Defaults to localhost.
         port : int
@@ -559,13 +559,7 @@ class FlureeClient:
                    Validity in seconda of the signature.
         sig_fuel : int
                    Not sure what this is for, consult FlureeDB documentation for info.
-        dryrun : bool
-                  Don't use HTTP, simply print queries/transactions instead
 
-        Raises
-        ------
-        FlureeHalfCredentials
-            If masterkey is specified but auth_address isn't, or the other way around.
         """
         self.host = host
         self.port = port
@@ -575,13 +569,10 @@ class FlureeClient:
         if https and not ssl_verify:
             self.ssl_verify_disabled = True
         self.signer = None
-        if masterkey and auth_address:
-            self.signer = DbSigner(masterkey, auth_address, None, sig_validity, sig_fuel)
-        if masterkey and not auth_address or auth_address and not masterkey:
-            raise FlureeHalfCredentials("masterkey and auth_address should either both be specified, or neither")
+        if masterkey:
+            self.signer = DbSigner(masterkey, None, sig_validity, sig_fuel)
         self.session = None
-        if not dryrun:
-            self.session = aiohttp.ClientSession()
+        self.session = aiohttp.ClientSession()
         self.known_endpoints = set(["dbs",
                                     "new_db",
                                     "delete_db",
@@ -591,7 +582,7 @@ class FlureeClient:
                                     "new_keys"])
         self.unsigned_endpoints = set(["dbs", "health", "new_keys"])
         self.use_get = set(["health", "new_keys"])
-        self.required = dict()
+        self.required = {}
         self.required["new_db"] = set(["db_id"])
         self.required["delete_db"] = set(["db_id"])
         self.required["add_server"] = set(["server"])
@@ -737,15 +728,15 @@ class FlureeClient:
             Itteratable object with databases per network.
         """
         databases = await self.dbs()
-        optionsmap = dict()
+        optionsmap = {}
         for pair in databases:
             network = pair[0]
             database = pair[1]
             if network not in optionsmap:
                 optionsmap[network] = set()
             optionsmap[network].add(database)
-        for key in optionsmap:
-            yield _Network(self, key, optionsmap[key])
+        for key, item in optionsmap.items():
+            yield _Network(self, key, item)
 
     async def close_session(self):
         """Close HTTP(S) session to FlureeDB"""
@@ -758,23 +749,19 @@ class _FlureeDbClient:
     """Basic asynchonous client for FlureeDB representing a particular database on FlureeDB"""
     def __init__(self,
                  privkey,
-                 auth_address,
                  database,
                  host="localhost",
                  port=8080,
                  https=False,
                  ssl_verify=True,
                  sig_validity=120,
-                 sig_fuel=1000,
-                 dryrun=False):
+                 sig_fuel=1000):
         """Constructor
 
         Parameters
         ----------
         privkey : string
                   Hex or base58 encoded signing key
-        auth_address : string
-                       key-id of the signing key
         database : string
                    net/db string of the flureeDB database
         host : string
@@ -789,32 +776,30 @@ class _FlureeDbClient:
                    Validity in seconda of the signature.
         sig_fuel : int
                    Not sure what this is for, consult FlureeDB documentation for info.
-        dryrun : bool
-                 Don't use HTTP, simply print queries/transactions instead
         """
         self.database = database
         self.host = host
         self.port = port
         self.https = https
         self.ssl_verify_disabled = False
-        self.monitor = dict()
-        self.monitor["listeners"] = dict()
+        self.monitor = {}
+        self.monitor["listeners"] = {}
         self.monitor["running"] = False
         self.monitor["next"] = None
         self.monitor["rewind"] = 0
         self.monitor["on_block_processed"] = None
-        self.monitor["predicate_map"] = dict()
+        self.monitor["predicate_map"] = {}
+        self.monitor["predicate_map_multi"] = {}
         self.monitor["predicate_map_block"] = 0
         self.monitor["lastblock_instant"] = None
-        self.monitor["instant_monitors"] = list()
+        self.monitor["instant_monitors"] = []
         if https and not ssl_verify:
             self.ssl_verify_disabled = True
         self.signer = None
-        if privkey and auth_address:
-            self.signer = DbSigner(privkey, auth_address, database, sig_validity, sig_fuel)
+        if privkey:
+            self.signer = DbSigner(privkey, database, sig_validity, sig_fuel)
         self.session = None
-        if not dryrun:
-            self.session = aiohttp.ClientSession()
+        self.session = aiohttp.ClientSession()
         self.known_endpoints = set(["snapshot",
                                     "list_snapshots",
                                     "export",
@@ -837,7 +822,14 @@ class _FlureeDbClient:
                                     "storage",
                                     "pw"])
         self.pw_endpoints = set(["generate", "renew", "login"])
-        self.implemented = set(["query", "flureeql", "block", "command", "ledger_stats", "list_snapshots", "snapshot"])
+        self.implemented = set(["query",
+                                "flureeql",
+                                "block",
+                                "command",
+                                "ledger_stats",
+                                "list_snapshots",
+                                "snapshot",
+                                "multi_query"])
 
     def monitor_init(self, on_block_processed, start_block=None, rewind=0, always_query_object=False, start_instant=None):
         """Set the basic variables for a fluree block event monitor run
@@ -893,7 +885,7 @@ class _FlureeDbClient:
         assert isinstance(collection, str)
         assert callable(callback)
         if collection not in self.monitor["listeners"]:
-            self.monitor["listeners"][collection] = dict()
+            self.monitor["listeners"][collection] = {}
         if "C" not in self.monitor["listeners"][collection]:
             self.monitor["listeners"][collection]["C"] = set()
         self.monitor["listeners"][collection]["C"].add(callback)
@@ -913,7 +905,7 @@ class _FlureeDbClient:
         assert isinstance(collection, str)
         assert callable(callback)
         if collection not in self.monitor["listeners"]:
-            self.monitor["listeners"][collection] = dict()
+            self.monitor["listeners"][collection] = {}
         if "D" not in self.monitor["listeners"][collection]:
             self.monitor["listeners"][collection]["D"] = set()
         self.monitor["listeners"][collection]["D"].add(callback)
@@ -934,7 +926,7 @@ class _FlureeDbClient:
         assert isinstance(collection, str)
         assert callable(callback)
         if collection not in self.monitor["listeners"]:
-            self.monitor["listeners"][collection] = dict()
+            self.monitor["listeners"][collection] = {}
         if "U" not in self.monitor["listeners"][collection]:
             self.monitor["listeners"][collection]["U"] = set()
         self.monitor["listeners"][collection]["U"].add(callback)
@@ -984,17 +976,23 @@ class _FlureeDbClient:
         """
         if block is not None:
             if self.monitor["predicate_map_block"] != block:
-                predicates = await self.flureeql.query(select=["id", "name"], ffrom="_predicate", block=block)
+                predicates = await self.flureeql.query(select=["name", "multi"], ffrom="_predicate", block=block)
                 self.monitor["predicate_map_block"] = block
             else:
                 predicates = None
         else:
-            predicates = await self.flureeql.query(select=["id", "name"], ffrom="_predicate")
+            predicates = await self.flureeql.query(select=["name", "multi"], ffrom="_predicate")
         if predicates is not None:
-            predicate = dict()
+            predicate = {}
+            is_multi = {}
             for pred in predicates:
                 predicate[pred["_id"]] = pred["name"]
+                if "multi" in pred:
+                    is_multi[pred["name"]] = pred["multi"]
+                else:
+                    is_multi[pred["name"]] = False
             self.monitor["predicate_map"] = predicate
+            self.monitor["predicate_map_multi"] = is_multi
 
     async def _find_start_block(self):
         """Find the start block
@@ -1077,7 +1075,7 @@ class _FlureeDbClient:
             Raised when an unknown predicate id is detected.
         """
         has_predicate_updates = False
-        grouped = dict()
+        grouped = {}
         for flake in block_data[0]["flakes"]:
             predno = flake[1]
             # Patch numeric predicates to textual ones.
@@ -1087,11 +1085,13 @@ class _FlureeDbClient:
                 raise FlureeUnexpectedPredicateNumber("Need a restart after new predicates are added to the database")
             # Group the flakes together by object.
             if not flake[0] in grouped:
-                grouped[flake[0]] = list()
+                grouped[flake[0]] = []
             grouped[flake[0]].append(flake)
+        # pylint: disable=consider-using-dict-items
         for obj in grouped:
             if grouped[obj][0][1].split("/")[0] == "_predicate":
                 has_predicate_updates = True
+        # pylint: enable=consider-using-dict-items
         if has_predicate_updates:
             await self._build_predicates_map(blockno)
         return grouped
@@ -1189,27 +1189,28 @@ class _FlureeDbClient:
             object id to operation map.
         """
         # pylint: disable=too-many-nested-blocks, too-many-branches
-        obj_tx = dict()
+        obj_tx = {}
         if tempids:
             for tmp_id in tempids:
                 real_id = tempids[tmp_id]
-                counters = dict()
-                for operation in operations:
-                    if isinstance(operation, dict) and "_id" in operation:
-                        if isinstance(operation["_id"], str):
-                            if operation["_id"] == tmp_id:
-                                obj_tx[real_id] = operation
-                            if operation["_id"] not in counters:
-                                counters[operation["_id"]] = 0
-                            counters[operation["_id"]] += 1
-                            altname = operation["_id"] + "$" + str(counters[operation["_id"]])
-                            if altname == tmp_id:
-                                obj_tx[real_id] = operation
-                        elif isinstance(operation["_id"], list):
-                            if len(operation["_id"]) == 2:
-                                txid = '["' + operation["_id"][0] + '" "' + operation["_id"][1] + '"]'
-                                if txid == tmp_id:
+                counters = {}
+                if isinstance(real_id, int):
+                    for operation in operations:
+                        if isinstance(operation, dict) and "_id" in operation:
+                            if isinstance(operation["_id"], str):
+                                if operation["_id"] == tmp_id:
                                     obj_tx[real_id] = operation
+                                if operation["_id"] not in counters:
+                                    counters[operation["_id"]] = 0
+                                counters[operation["_id"]] += 1
+                                altname = operation["_id"] + "$" + str(counters[operation["_id"]])
+                                if altname == tmp_id:
+                                    obj_tx[real_id] = operation
+                            elif isinstance(operation["_id"], list):
+                                if len(operation["_id"]) == 2:
+                                    txid = '["' + operation["_id"][0] + '" "' + operation["_id"][1] + '"]'
+                                    if txid == tmp_id:
+                                        obj_tx[real_id] = operation
         if len(operations) == 1:
             obj_tx[""] = operations[0]
         for operation in operations:
@@ -1274,8 +1275,8 @@ class _FlureeDbClient:
             await self._build_predicates_map(blockno)
             grouped = await self._group_block_flakes(block_data, blockno)
         # Distill new ones using _tx/tempids
-        obj_tx = dict()
-        block_meta = dict()
+        obj_tx = {}
+        block_meta = {}
         for obj in grouped:
             transactions = None
             tempids = None
@@ -1320,11 +1321,15 @@ class _FlureeDbClient:
             operation = obj_tx[""]
         has_true = False
         has_false = False
+        has_multi = False
         for flake in obj:
             if flake[4]:
                 has_true = True
             else:
                 has_false = True
+            if flake[1] in self.monitor["predicate_map_multi"]:
+                if self.monitor["predicate_map_multi"][flake[1]]:
+                    has_multi = True
         if self.monitor["always_query_object"]:
             previous = await self.flureeql.query(select=["*"], ffrom=obj[0][0], block=blockno-1)
             if previous:
@@ -1343,14 +1348,17 @@ class _FlureeDbClient:
                 action = "delete"
             else:
                 action = "update"
-        if operation and "_action" in operation and operation["_action"] != "upsert":
+        if operation and "_action" in operation and operation["_action"] != "upsert" and not has_multi:
             action = operation["_action"]
         if action is None and has_true and has_false:
             action = "update"
         if action is None and operation and "_id" in operation and isinstance(operation["_id"], str):
             action = "insert"
         if action is None and operation and has_false and not has_true:
-            action = "delete"
+            if len(obj) == 1 and has_multi:
+                action = "update"
+            else:
+                action = "delete"
         if action is None and has_true and not has_false:
             if blockno > 1:
                 previous = await self.flureeql.query(select=["*"], ffrom=obj[0][0], block=blockno-1)
@@ -1464,7 +1472,13 @@ class _FlureeDbClient:
                     raise RuntimeError("Too many errors from ledger_stats call")
 
     async def ready(self):
-        """Awaitable that polls the database untill the schema contains collections"""
+        """Awaitable that polls the database untill the schema contains collections
+
+        Raises
+        ------
+        FlureeHttpError
+            When the error from FlureeDB is db/invalid-auth
+        """
         while True:
             try:
 
@@ -1473,8 +1487,11 @@ class _FlureeDbClient:
                     ffrom="_collection"
                 )
                 return
-            except FlureeHttpError:
-                await asyncio.sleep(0.1)
+            except FlureeHttpError as ex:
+                result = json.loads(ex.args[0])
+                if result["error"] == "db/invalid-auth":
+                    raise ex
+                await asyncio.sleep(2)
 
     async def __aexit__(self, exc_type, exc, traceback):
         await self.close_session()
@@ -1584,26 +1601,23 @@ class _FlureeDbClient:
                 FlureeHttpError
                     When HTTP status from fluree server is anything other than 200
                 """
-                if self.session:
-                    if self.ssl_verify_disabled:
-                        async with self.session.post(self.url, data=body, headers=headers, ssl=False) as resp:
-                            if resp.status != 200:
-                                raise FlureeHttpError(await resp.text(), resp.status)
-                            return await resp.text()
-                    else:
-                        async with self.session.post(self.url, data=body, headers=headers) as resp:
-                            if resp.status != 200:
-                                raise FlureeHttpError(await resp.text(), resp.status)
-                            return await resp.text()
+                if self.ssl_verify_disabled:
+                    async with self.session.post(self.url, data=body, headers=headers, ssl=False) as resp:
+                        if resp.status != 200:
+                            raise FlureeHttpError(await resp.text(), resp.status)
+                        return await resp.text()
                 else:
-                    return json.loads(_dryrun("POST", self.url, headers, body))
+                    async with self.session.post(self.url, data=body, headers=headers) as resp:
+                        if resp.status != 200:
+                            raise FlureeHttpError(await resp.text(), resp.status)
+                        return await resp.text()
 
             async def header_signed(self, query_body):
                 """Do a HTTP query using headers for signing
 
                 Parameters
                 ----------
-                query_body : dict/list/string
+                query_body : any
                        query body to sign using headers.
 
                 Returns
@@ -1624,6 +1638,8 @@ class _FlureeDbClient:
                 ----------
                 transact_obj : list
                        transaction to sign using body envelope.
+                deps: dict
+                    FlureeDb debs
 
                 Returns
                 -------
@@ -1646,6 +1662,82 @@ class _FlureeDbClient:
                 """
                 return await self._post_body_with_headers(None, None)
 
+        class FlureeQlEndpointMulti:
+            """Endpoint for JSON based (FlureeQl) multi-queries"""
+            def __init__(self, client, ssl_verify_disabled, raw=None):
+                """Constructor
+
+                Parameters
+                ----------
+                client: object
+                        The wrapping _FlureeDbClient
+
+                ssl_verify_disabled: bool
+                    When using https, don't validata ssl certs.
+
+                raw: dict
+                    The whole raw multiquery
+                """
+                self.stringendpoint = _StringEndpoint("multi_query", client, ssl_verify_disabled)
+                if raw:
+                    self.multi_query = raw
+                else:
+                    self.multi_query = {}
+
+            def __call__(self, raw=None):
+                """Invoke as function object.
+
+                Parameters
+                ----------
+                raw: dict
+                    The whole raw multiquery
+
+                Returns
+                -------
+                FlureeQlEndpointMulti
+                    Pointer to self
+                """
+                if raw is not None:
+                    self.multi_query = raw
+                return self
+
+            def __dir__(self):
+                """Dir function for class
+
+                Returns
+                -------
+                list
+                    List of defined (pseudo) attributes
+                """
+                return ["__call__", "__dir__", "__init__"]
+
+            def __getattr__(self, method):
+                """query
+
+                Parameters
+                ----------
+                method : string
+                         subquery name
+
+                Returns
+                -------
+                _FlureeQlSubQuery
+                    Helper class for creating FlureeQl multi-queries.
+
+                """
+                return _FlureeQlSubQuery(self, method)
+
+            async def query(self):
+                """Do the actual multi-query
+
+                Returns
+                -------
+                dict
+                    The result from the mult-query
+                """
+                return_body = await self.stringendpoint.header_signed(self.multi_query)
+                return json.loads(return_body)
+
         class FlureeQlEndpoint:
             """Endpoint for JSON based (FlureeQl) queries"""
             def __init__(self, api_endpoint, client, ssl_verify_disabled):
@@ -1740,6 +1832,11 @@ class _FlureeDbClient:
                 ----------
                 transaction_obj : list
                                Transaction list
+                deps: dict
+                    FlureeDb debs
+
+                do_await: bool
+                    Do we wait for the transaction to complete, or do we fire and forget?
 
                 Returns
                 -------
@@ -1798,6 +1895,8 @@ class _FlureeDbClient:
             if self.signer is None:
                 raise FlureeKeyRequired("Command endpoint not supported in open-API mode. privkey required!")
             return CommandEndpoint(api_endpoint, self, self.ssl_verify_disabled)
+        if api_endpoint in ["multi_query"]:
+            return FlureeQlEndpointMulti(self, self.ssl_verify_disabled)
         if api_endpoint == 'ledger_stats':
             return LedgerStatsEndpoint(self, self.ssl_verify_disabled)
         return FlureeQlEndpoint(api_endpoint, self, self.ssl_verify_disabled)
